@@ -15,8 +15,7 @@ import (
 // Generates a response to the status-get command including two status
 // structures, one for DHCPv4 and one for DHCPv6.
 func mockGetStatusWithHA(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4", "dhcp6")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4", "dhcp6"}, nil)
 	var json string
 	switch callNo {
 	case 0:
@@ -105,8 +104,7 @@ func mockGetStatusWithHA(callNo int, cmdResponses []interface{}) {
 // structures, one for DHCPv4 and one for DHCPv6. Format supported by
 // Kea 1.7.8 onwards.
 func mockGetStatusWithHA178(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4", "dhcp6")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4", "dhcp6"}, nil)
 	var json string
 	switch callNo {
 	case 0:
@@ -221,8 +219,7 @@ func mockGetStatusWithHA178(callNo int, cmdResponses []interface{}) {
 // Generate test response to status-get command including status of the
 // HA pair doing load balancing.
 func mockGetStatusLoadBalancing(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4"}, nil)
 	json := `[
         {
             "result": 0,
@@ -255,8 +252,7 @@ func mockGetStatusLoadBalancing(callNo int, cmdResponses []interface{}) {
 // Generate test response to status-get command including status of the
 // HA pair doing load balancing. Format supported by Kea 1.7.8 onwards.
 func mockGetStatusLoadBalancing178(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4"}, nil)
 	json := `[
         {
             "result": 0,
@@ -299,8 +295,7 @@ func mockGetStatusLoadBalancing178(callNo int, cmdResponses []interface{}) {
 // Generates test response to status-get command lacking a status of the
 // HA pair.
 func mockGetStatusNoHA(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4"}, nil)
 	json := `[
         {
             "result": 0,
@@ -318,8 +313,7 @@ func mockGetStatusNoHA(callNo int, cmdResponses []interface{}) {
 // Generates test response to status-get command indicating an error and
 // lacking arguments.
 func mockGetStatusError(callNo int, cmdResponses []interface{}) {
-	daemons, _ := keactrl.NewDaemons("dhcp4")
-	command, _ := keactrl.NewCommand("status-get", daemons, nil)
+	command := keactrl.NewCommand("status-get", []string{"dhcp4"}, nil)
 	json := `[
         {
             "result": 1,
