@@ -49,6 +49,11 @@ func TestNewSharedNetworkFromKea(t *testing.T) {
 			{
 				"id":     1,
 				"subnet": "2001:db8:2::/64",
+				"reservations": []interface{}{
+					map[string]interface{}{
+						"hw-address": "01:02:03:04:05:06",
+					},
+				},
 			},
 			{
 				"id":     2,
@@ -56,8 +61,8 @@ func TestNewSharedNetworkFromKea(t *testing.T) {
 			},
 		},
 	}
-
-	parsedNetwork, err := NewSharedNetworkFromKea(&rawNetwork, 6)
+	daemon := NewKeaDaemon(DaemonNameDHCPv6, true)
+	parsedNetwork, err := NewSharedNetworkFromKea(&rawNetwork, 6, daemon, HostDataSourceConfig)
 	require.NoError(t, err)
 	require.NotNil(t, parsedNetwork)
 	require.Equal(t, "foo", parsedNetwork.Name)
@@ -68,6 +73,8 @@ func TestNewSharedNetworkFromKea(t *testing.T) {
 	require.Equal(t, "2001:db8:2::/64", parsedNetwork.Subnets[0].Prefix)
 	require.Zero(t, parsedNetwork.Subnets[1].ID)
 	require.Equal(t, "2001:db8:1::/64", parsedNetwork.Subnets[1].Prefix)
+
+	require.Len(t, parsedNetwork.Subnets[0].Hosts, 1)
 }
 
 // Test that subnets within a shared network are verified to catch
@@ -89,7 +96,8 @@ func TestNewSharedNetworkFromKeaFamilyClash(t *testing.T) {
 		},
 	}
 
-	parsedNetwork, err := NewSharedNetworkFromKea(&rawNetwork, 4)
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	parsedNetwork, err := NewSharedNetworkFromKea(&rawNetwork, 4, daemon, HostDataSourceConfig)
 	require.Error(t, err)
 	require.Nil(t, parsedNetwork)
 }
@@ -138,7 +146,9 @@ func TestNewSubnetFromKea(t *testing.T) {
 		},
 	}
 
-	parsedSubnet, err := NewSubnetFromKea(&rawSubnet)
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 234
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
 	require.NoError(t, err)
 	require.NotNil(t, parsedSubnet)
 	require.Zero(t, parsedSubnet.ID)
@@ -163,6 +173,96 @@ func TestNewSubnetFromKea(t *testing.T) {
 	require.Equal(t, "2001:db8:1::2", parsedSubnet.Hosts[0].IPReservations[1].Address)
 	require.Equal(t, "3000:1::/64", parsedSubnet.Hosts[0].IPReservations[2].Address)
 	require.Equal(t, "3000:2::/64", parsedSubnet.Hosts[0].IPReservations[3].Address)
+
+	require.Len(t, parsedSubnet.Hosts[0].LocalHosts, 1)
+	require.Equal(t, parsedSubnet.Hosts[0].ID, parsedSubnet.Hosts[0].LocalHosts[0].HostID)
+	require.EqualValues(t, 234, parsedSubnet.Hosts[0].LocalHosts[0].DaemonID)
+	require.Equal(t, HostDataSourceConfig, parsedSubnet.Hosts[0].LocalHosts[0].DataSource)
+}
+
+// Test that the error is returned when the subnet prefix is invalid.
+func TestNewSubnetFromKeaWithInvalidPrefix(t *testing.T) {
+	// Arrange
+	rawSubnet := map[string]interface{}{
+		"subnet": "invalid",
+	}
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 42
+
+	// Act
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
+
+	// Assert
+	require.Error(t, err)
+	require.Nil(t, parsedSubnet)
+}
+
+// Test that the default mask is added to IPv4 subnet prefix if missing.
+func TestNewSubnetFromKeaWithDefaultIPv4PrefixMask(t *testing.T) {
+	// Arrange
+	rawSubnet := map[string]interface{}{
+		"subnet": "10.42.42.42",
+	}
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 42
+
+	// Act
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, "10.42.42.42/32", parsedSubnet.Prefix)
+}
+
+// Test that the default mask is added to IPv6 subnet prefix if missing.
+func TestNewSubnetFromKeaWithDefaultIPv6PrefixMask(t *testing.T) {
+	// Arrange
+	rawSubnet := map[string]interface{}{
+		"subnet": "fe80::42",
+	}
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 42
+
+	// Act
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, "fe80::42/128", parsedSubnet.Prefix)
+}
+
+// Test that the IPv4 subnet prefix is converted from non-canonical to canonical form.
+func TestNewSubnetFromKeaWithNonCanonicalIPv4Prefix(t *testing.T) {
+	// Arrange
+	rawSubnet := map[string]interface{}{
+		"subnet": "10.42.42.42/8",
+	}
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 42
+
+	// Act
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, "10.0.0.0/8", parsedSubnet.Prefix)
+}
+
+// Test that the IPv6 subnet prefix is converted from non-canonical to canonical form.
+func TestNewSubnetFromKeaWithNonCanonicalIPv6Prefix(t *testing.T) {
+	// Arrange
+	rawSubnet := map[string]interface{}{
+		"subnet": "2001:db8:1::42/64",
+	}
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 42
+
+	// Act
+	parsedSubnet, err := NewSubnetFromKea(&rawSubnet, daemon, HostDataSourceConfig)
+
+	// Assert
+	require.NoError(t, err)
+	require.EqualValues(t, "2001:db8:1::/64", parsedSubnet.Prefix)
 }
 
 // Verifies that the host instance can be created by parsing Kea
@@ -179,9 +279,21 @@ func TestNewHostFromKea(t *testing.T) {
 			"3000:2::/64",
 		},
 		"hostname": "hostname.example.org",
+		"option-data": []interface{}{
+			map[string]interface{}{
+				"always-send": true,
+				"code":        23,
+				"csv-format":  true,
+				"data":        "2001:db8:1::15,2001:db8:1::16",
+				"name":        "dns-servers",
+				"space":       "dhcp6",
+			},
+		},
 	}
 
-	parsedHost, err := NewHostFromKea(&rawHost)
+	daemon := NewKeaDaemon(DaemonNameDHCPv4, true)
+	daemon.ID = 123
+	parsedHost, err := NewHostFromKea(&rawHost, daemon, HostDataSourceConfig)
 	require.NoError(t, err)
 	require.NotNil(t, parsedHost)
 
@@ -193,6 +305,24 @@ func TestNewHostFromKea(t *testing.T) {
 	require.Equal(t, "3000:1::/64", parsedHost.IPReservations[2].Address)
 	require.Equal(t, "3000:2::/64", parsedHost.IPReservations[3].Address)
 	require.Equal(t, "hostname.example.org", parsedHost.Hostname)
+	require.Len(t, parsedHost.LocalHosts, 1)
+	require.Equal(t, parsedHost.ID, parsedHost.LocalHosts[0].HostID)
+	require.EqualValues(t, 123, parsedHost.LocalHosts[0].DaemonID)
+	require.Equal(t, HostDataSourceConfig, parsedHost.LocalHosts[0].DataSource)
+
+	// DHCP options
+	require.Len(t, parsedHost.LocalHosts[0].DHCPOptionSet, 1)
+	require.True(t, parsedHost.LocalHosts[0].DHCPOptionSet[0].AlwaysSend)
+	require.EqualValues(t, 23, parsedHost.LocalHosts[0].DHCPOptionSet[0].Code)
+	require.Equal(t, "dns-servers", parsedHost.LocalHosts[0].DHCPOptionSet[0].Name)
+	require.Equal(t, "dhcp6", parsedHost.LocalHosts[0].DHCPOptionSet[0].Space)
+	require.Len(t, parsedHost.LocalHosts[0].DHCPOptionSet[0].Fields, 2)
+	require.Equal(t, "ipv6-address", parsedHost.LocalHosts[0].DHCPOptionSet[0].Fields[0].FieldType)
+	require.Len(t, parsedHost.LocalHosts[0].DHCPOptionSet[0].Fields[0].Values, 1)
+	require.Equal(t, "2001:db8:1::15", parsedHost.LocalHosts[0].DHCPOptionSet[0].Fields[0].Values[0])
+
+	// Make sure the hash is computed.
+	require.NotEmpty(t, parsedHost.LocalHosts[0].DHCPOptionSetHash)
 }
 
 // Test that log targets can be created from parsed Kea logger config.
