@@ -25,6 +25,7 @@ import { MessagesModule } from 'primeng/messages'
 import { TabMenuModule } from 'primeng/tabmenu'
 import { MenuModule } from 'primeng/menu'
 import { InplaceModule } from 'primeng/inplace'
+import { ChipModule } from 'primeng/chip'
 import { ProgressSpinnerModule } from 'primeng/progressspinner'
 import { TooltipModule } from 'primeng/tooltip'
 import { PasswordModule } from 'primeng/password'
@@ -43,6 +44,7 @@ import { MultiSelectModule } from 'primeng/multiselect'
 import { CheckboxModule } from 'primeng/checkbox'
 import { ConfirmDialogModule } from 'primeng/confirmdialog'
 import { InputTextareaModule } from 'primeng/inputtextarea'
+import { TreeModule } from 'primeng/tree'
 
 // Generated API modules
 import { ApiModule, BASE_PATH, Configuration, ConfigurationParameters } from './backend'
@@ -95,6 +97,10 @@ import { AppOverviewComponent } from './app-overview/app-overview.component'
 import { HostFormComponent } from './host-form/host-form.component'
 import { DhcpOptionFormComponent } from './dhcp-option-form/dhcp-option-form.component'
 import { DhcpOptionSetFormComponent } from './dhcp-option-set-form/dhcp-option-set-form.component'
+import { ConfigCheckerPreferenceUpdaterComponent } from './config-checker-preference-updater/config-checker-preference-updater.component'
+import { ConfigCheckerPreferencePickerComponent } from './config-checker-preference-picker/config-checker-preference-picker.component'
+import { ConfigCheckerPreferencePageComponent } from './config-checker-preference-page/config-checker-preference-page.component'
+import { DhcpOptionSetViewComponent } from './dhcp-option-set-view/dhcp-option-set-view.component'
 
 export function cfgFactory() {
     const params: ConfigurationParameters = {
@@ -149,6 +155,10 @@ export function cfgFactory() {
         HostFormComponent,
         DhcpOptionFormComponent,
         DhcpOptionSetFormComponent,
+        DhcpOptionSetViewComponent,
+        ConfigCheckerPreferencePickerComponent,
+        ConfigCheckerPreferenceUpdaterComponent,
+        ConfigCheckerPreferencePageComponent,
     ],
     imports: [
         BrowserModule,
@@ -195,6 +205,8 @@ export function cfgFactory() {
         CheckboxModule,
         ConfirmDialogModule,
         InputTextareaModule,
+        TreeModule,
+        ChipModule,
     ],
     providers: [
         {
